@@ -21,4 +21,4 @@ def group_admin(user):
     """
     Returns True or False based on if the user is an admin for any Groups
     """
-    return user.is_superuser or user.perms_on_any(Group, ['admin'])
\ No newline at end of file
+    return user.is_superuser or user.has_any_perms(Group, ['admin'])
\ No newline at end of file
